from StringIO import StringIO
from ftw.book.interfaces import IBook
from ftw.book.latex.utils import ImageLaTeXGenerator
from ftw.book.latex.utils import get_latex_heading
from ftw.book.latex.utils import get_raw_image_data
from ftw.pdfgenerator.html2latex.utils import generate_manual_caption
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.testing import MockTestCase
from mocker import ANY
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.app.component.hooks import setSite
from zope.component import getGlobalSiteManager
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.interface import alsoProvides


class TestLatexHeading(MockTestCase):

    def mock_inject_settings(self, mock, **settings):
        schema = self.stub()
        self.expect(mock.Schema()).result(schema).count(0, None)

        for fieldname, value in settings.items():
            self.expect(schema.getField(fieldname).get(mock)).result(value)

        self.expect(schema.getField(ANY)).result(None)

        return schema

    def test_latex_heading_of_primary_chapter(self):
        chapter = self.mocker.mock()
        self.set_parent(chapter, self.stub_interface(IBook))
        self.expect(chapter.pretty_title_or_id()).result('My Chapter')
        self.mock_inject_settings(chapter)

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(
            layout.get_converter().convert('My Chapter')).result(
            'My Chapter')

        self.replay()

        self.assertEquals(get_latex_heading(chapter, layout),
                          '\\chapter{My Chapter}\n')

    def test_hide_from_toc_setting_from_latex_injection(self):
        textblock = self.stub()
        self.set_parent(textblock, self.stub_interface(IBook))
        self.expect(textblock.pretty_title_or_id()).result('My Textblock')

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(layout.get_converter().convert('My Textblock')).result(
            'My Textblock')

        with self.mocker.order():
            self.mock_inject_settings(textblock, hideFromTOC=True)
            self.mock_inject_settings(textblock, hideFromTOC=True)
            self.mock_inject_settings(textblock, hideFromTOC=False)

        self.replay()

        self.assertEquals(get_latex_heading(textblock, layout),
                          '\\chapter*{My Textblock}\n')

        # Passed "toc" argument should take precedence
        self.assertEquals(get_latex_heading(textblock, layout, toc=True),
                          '\\chapter{My Textblock}\n')

        self.assertEquals(get_latex_heading(textblock, layout, toc=False),
                          '\\chapter*{My Textblock}\n')

    def test_latex_heading_of_primary_chapter_without_toc(self):
        chapter = self.mocker.mock()
        self.set_parent(chapter, self.stub_interface(IBook))
        self.expect(chapter.pretty_title_or_id()).result('My Chapter')
        self.mock_inject_settings(chapter)

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(layout.get_converter().convert('My Chapter')
                    ).result('My Chapter')

        self.replay()

        self.assertEquals(get_latex_heading(chapter, layout, toc=False),
                          '\\chapter*{My Chapter}\n')

    def test_latex_heading_of_third_level_chapter(self):
        chapter1 = self.mocker.mock()
        self.set_parent(chapter1, self.stub_interface(IBook))

        chapter2 = self.mocker.mock()
        self.set_parent(chapter2, chapter1)

        chapter3 = self.mocker.mock()
        self.set_parent(chapter3, chapter2)
        self.expect(chapter3.pretty_title_or_id()).result('Sub chapter')
        self.mock_inject_settings(chapter3)

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(layout.get_converter().convert('Sub chapter')
                    ).result('Sub chapter')

        self.replay()

        self.assertEquals(get_latex_heading(chapter3, layout),
                          '\\subsection{Sub chapter}\n')

    def test_latex_heading_with_max_level_exceeded(self):
        book = self.providing_stub([IBook])

        # create 10 objects and use the last one
        obj = None
        previous = book
        for i in range(10):
            obj = self.mocker.mock()
            self.set_parent(obj, previous)
            previous = obj

        self.expect(obj.pretty_title_or_id()).result('the title')
        self.mock_inject_settings(obj)

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(layout.get_converter().convert('the title')
                    ).result('the title')

        self.replay()

        # subparagraph is the "smallest" heading..
        self.assertEquals(get_latex_heading(obj, layout),
                          '\\subparagraph{the title}\n')

    def test_not_within_book(self):
        chapter = self.mocker.mock()
        self.set_parent(chapter, self.stub_interface(INavigationRoot))
        self.expect(chapter.pretty_title_or_id()).result('Any chapter')
        self.mock_inject_settings(chapter)

        layout = self.stub_interface(ILaTeXLayout)
        self.expect(layout.get_converter().convert('Any chapter')
                    ).result('Any chapter')

        self.replay()

        self.assertEquals(get_latex_heading(chapter, layout),
                          '\\section{Any chapter}\n')


class TestImageLaTeXGenerator(MockTestCase):

    def setUp(self):
        super(TestImageLaTeXGenerator, self).setUp()

        self.image = self.create_dummy(
            get_size=lambda: 11, data='hello world')

        self.context = self.create_dummy(
            UID=lambda: 'XUID')

        setSite(self._create_site_with_request())

    def tearDown(self):
        setSite(None)

    def _create_site_with_request(self):
        request = self.create_dummy(getPreferredLanguages=lambda: [])
        alsoProvides(request, IUserPreferredLanguages)

        site = self.create_dummy(
            REQUEST=request,
            getSiteManager=getGlobalSiteManager)

        return site

    def test_no_image_layout(self):
        self.replay()

        generator = ImageLaTeXGenerator(None, None)
        self.assertEqual(generator(None, 'no-image'), '')

    def test_small_left_nonfloating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'small')

        self.assertEqual(
            latex,
            r'\includegraphics[width=0.25\linewidth]{XUID_image}')

    def test_small_left_nonfloating_caption(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('The Caption')).result(
            'THE CAPTION')
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'small', caption='The Caption')

        self.assertEqual(latex, '\n'.join((
                    r'\includegraphics[width=0.25\linewidth]{XUID_image}',
                    generate_manual_caption('THE CAPTION', 'figure'),
                    )))

    def test_small_left_floating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.use_package('wrapfig'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.use_package('checkheight'))
        builder = self.stub()
        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.build_directory).result('/tmp')
        self.expect(builder.add_file('checkheight.sty', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'small', floatable=True)

        self.assertEqual(latex, '\n'.join((
                    r'\checkheight{\includegraphics[width=0.35\linewidth]{XUID_image}}',
                    r'\begin{wrapfigure}{l}{0.25\linewidth}',
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    r'\end{wrapfigure}',
                    r'\hspace{0em}%%'
                    )))

    def test_small_left_floating_caption(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.use_package('wrapfig'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('The Caption')).result(
            'THE CAPTION')
        self.expect(layout.use_package('checkheight'))
        builder = self.stub()
        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.build_directory).result('/tmp')
        self.expect(builder.add_file('checkheight.sty', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'small', floatable=True,
                          caption='The Caption')

        self.assertEqual(latex, '\n'.join((
                    r'\checkheight{\includegraphics[width=0.35\linewidth]{XUID_image}}',
                    r'\begin{wrapfigure}{l}{0.25\linewidth}',
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    r'\caption{THE CAPTION}',
                    r'\end{wrapfigure}',
                    r'\hspace{0em}%%'
                    )))

    def test_middle_left_nonfloating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'middle')

        self.assertEqual(
            latex,
            r'\includegraphics[width=0.5\linewidth]{XUID_image}')

    def test_full(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'full')

        self.assertEqual(
            latex,
            r'\includegraphics[width=\linewidth]{XUID_image}')

    def test_fullwidth_does_not_float(self):
        """Using a floating area (wrapfigure) with a 100% width causes the
        following text to be floated over the image.
        Since in this case floating is not possible (the image takes 100%)
        it should switch to non-floating even when called with
        ``floatable=True``.
        """

        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('My Image')).result(
            'MY IMAGE')
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'full', floatable=True,
                          caption='My Image')

        self.assertEqual(latex, '\n'.join((
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    generate_manual_caption('MY IMAGE', 'figure'))))

    def test_middle_right_nonfloating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'middle-right')

        self.assertEqual(latex, '\n'.join((
                    r'\begin{flushright}',
                    r'\includegraphics[width=0.5\linewidth]{XUID_image}',
                    r'\end{flushright}',
                    )))

    def test_middle_right_nonfloating_caption(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('The Caption')).result(
            'THE CAPTION')
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'middle-right',
                          caption='The Caption')

        self.assertEqual(latex, '\n'.join((
                    r'\begin{flushright}',
                    r'\includegraphics[width=0.5\linewidth]{XUID_image}',
                    generate_manual_caption('THE CAPTION', 'figure'),
                    r'\end{flushright}',
                    )))

    def test_middle_right_floating_caption(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.use_package('wrapfig'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('The Caption')).result(
            'THE CAPTION')
        self.expect(layout.use_package('checkheight'))
        builder = self.stub()
        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.build_directory).result('/tmp')
        self.expect(builder.add_file('checkheight.sty', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'middle-right', floatable=True,
                          caption='The Caption')

        self.assertEqual(latex, '\n'.join((
                    r'\checkheight{\includegraphics[width=0.6\linewidth]{XUID_image}}',
                    r'\begin{wrapfigure}{r}{0.5\linewidth}',
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    r'\caption{THE CAPTION}',
                    r'\end{wrapfigure}',
                    r'\hspace{0em}%%'
                    )))

    def test_small_right_nonfloating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'small-right')

        self.assertEqual(latex, '\n'.join((
                    r'\begin{flushright}',
                    r'\includegraphics[width=0.25\linewidth]{XUID_image}',
                    r'\end{flushright}',
                    )))

    def test_unkown_layout(self):
        # The includegraphics options should never have an empty width
        # option, like [image=], so the default is be 100% when the layout
        # is not recognized.
        # Having a [image=] will make pdflatex hang and this will block the
        # zope thread.

        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator(self.image, 'fancy-unkown')

        self.assertEqual(latex, '\n'.join((
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    )))

    def test_render_floating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.use_package('wrapfig'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.expect(layout.get_converter().convert('The Caption')).result(
            'THE CAPTION')
        self.expect(layout.use_package('checkheight'))
        builder = self.stub()
        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.build_directory).result('/tmp')
        self.expect(builder.add_file('checkheight.sty', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator.render(self.image, '0.56', 'c',
                                 floatable=True,
                                 caption='The Caption')

        self.assertEqual(latex, '\n'.join((
                    r'\checkheight{\includegraphics[width=0.66\linewidth]{XUID_image}}',
                    r'\begin{wrapfigure}{c}{0.56\linewidth}',
                    r'\includegraphics[width=\linewidth]{XUID_image}',
                    r'\caption{THE CAPTION}',
                    r'\end{wrapfigure}',
                    r'\hspace{0em}%%'
                    )))

    def test_render_nofloating(self):
        layout = self.mock_interface(ILaTeXLayout)
        self.expect(layout.use_package('graphicx'))
        self.expect(layout.get_builder().add_file('XUID_image.jpg', ANY))
        self.replay()

        generator = ImageLaTeXGenerator(self.context, layout)
        latex = generator.render(self.image, '0.56', 'c')

        self.assertEqual(latex, '\n'.join((
                    r'\begin{center}',
                    r'\includegraphics[width=0.56\linewidth]{XUID_image}',
                    r'\end{center}',
                    )))


class TestGetRawImageData(MockTestCase):

    def test_get_raw_image_data(self):
        already_raw = 'Image data'
        self.assertEquals(get_raw_image_data(already_raw),
                          already_raw)

        image = self.create_dummy(data=StringIO('stream data'))
        self.assertEquals(get_raw_image_data(image), 'stream data')

        image2 = self.create_dummy(data='direct data')
        self.assertEquals(get_raw_image_data(image2), 'direct data')



from Products.ATContentTypes.interfaces.image import IATImage
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.html2latex.utils import generate_manual_caption
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.testing import MockTestCase
from simplelayout.base.interfaces import IBlockConfig
from zope.app.component.hooks import setSite
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.interface import alsoProvides


class TestImageLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestImageLaTeXView, self).setUp()
        setSite(self._create_site_with_request())

    def tearDown(self):
        setSite(None)

    def _create_site_with_request(self):
        request = self.create_dummy(getPreferredLanguages=lambda: [])
        alsoProvides(request, IUserPreferredLanguages)

        site = self.create_dummy(
            REQUEST=request,
            getSiteManager=getGlobalSiteManager)

        return site

    def create_mocks(self, image_layout,
                     description, uid):

        # mock is not yet in replay mode, so we use another layout dummy..
        layout_obj = self.create_dummy()
        alsoProvides(layout_obj, ILaTeXLayout)
        converter = getMultiAdapter((object(), object(), layout_obj),
                                    IHTML2LaTeXConverter)

        image = self.create_dummy(size=11,
                                  data='hello world')
        context = self.providing_stub([IATImage, IBlockConfig])
        request = self.create_dummy()
        layout = self.providing_mock([ILaTeXLayout])
        builder = self.mocker.mock()

        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.add_file('%s_image.jpg' % uid, image.data))

        self.expect(layout.get_converter()).result(converter).count(0, None)
        self.expect(layout.use_package('graphicx'))

        self.expect(context.getImage()).result(image).count(1, None)
        self.expect(context.image_layout).result(image_layout)
        self.expect(context.UID()).result(uid)
        self.expect(context.Description()).result(description)

        return context, request, layout, converter

    def test_no_latex_with_no_image(self):
        request = self.create_dummy()
        layout = self.create_dummy()

        context = self.providing_stub([IATImage, IBlockConfig])
        self.expect(context.Description()).result(None)
        self.expect(context.getImage()).result(object())
        self.expect(context.image_layout).result('no-image')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()
        self.assertEqual(latex, '')

    def test_latex_with_small_layout(self):
        context, request, layout, converter = self.create_mocks(
            'small', 'my description', '123')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertEqual(
            latex,
            '\n'.join([
                    r'\includegraphics[width=0.25\linewidth]{123_image}',
                    generate_manual_caption('my description', 'figure')]))

    def test_latex_with_middle_layout(self):
        context, request, layout, converter = self.create_mocks(
            'middle', 'the description', '3434')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertEqual(
            latex,
            '\n'.join([
                    r'\includegraphics[width=0.5\linewidth]{3434_image}',
                    generate_manual_caption('the description', 'figure'),
                    ]))

    def test_latex_with_full_layout(self):
        context, request, layout, converter = self.create_mocks(
            'full', 'description', '12full')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertEqual(
            latex,
            '\n'.join([
                    r'\includegraphics[width=\linewidth]{12full_image}',
                    generate_manual_caption('description', 'figure'),
                    ]))

    def test_latex_with_full_layout_no_description(self):
        context, request, layout, converter = self.create_mocks(
            'full', '', '123full')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertEqual(
            latex,
            '\n'.join([
                    r'\includegraphics[width=\linewidth]{123full_image}',
                    ]))

    def test_latex_with_middle_right_layout(self):
        context, request, layout, converter = self.create_mocks(
            'middle-right', 'description', '1mr')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertIn(r'\includegraphics[width=0.5\linewidth]{1mr_image}',
                      latex)

    def test_latex_with_middle_small_layout(self):
        context, request, layout, converter = self.create_mocks(
            'small-right', 'description', '1sr')

        self.replay()

        view = getMultiAdapter((context, request, layout))
        latex = view.render()

        self.assertIn(r'\includegraphics[width=0.25\linewidth]{1sr_image}',
                      latex)

    def test_default_width(self):
        # The includegraphics options should never have an empty width
        # option, like [image=], so the default is be 100% when the layout
        # is not recognized.
        # Having a [image=] will make pdflatex hang and this will block the
        # zope thread.

        textblock, request, layout, converter = self.create_mocks(
            'a really bad unkown layout', 'desc', '123')

        self.replay()

        view = getMultiAdapter((textblock, request, layout))
        latex = view.render()

        self.assertNotIn('[width=]', latex)
        self.assertIn(r'[width=\linewidth]', latex)



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from unittest2 import TestCase


class TestRemark(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.book = create(Builder('book').titled('The Book'))
        self.chapter = create(Builder('chapter')
                              .titled('The Chapter')
                              .within(self.book))

    @browsing
    def test_creating_remark(self, browser):
        browser.login().visit(self.chapter)
        factoriesmenu.add('Remark')

        browser.fill({
                'Title': 'The Remark',
                'Text': '<p>Some <b>body</b> text</p>'}).submit()
        self.assertEquals(self.chapter.absolute_url() + '/#the-remark',
                          browser.url)

        blocks = browser.css('.BlockOverallWrapper.remark')
        self.assertEquals(1, len(blocks),
                          'Expected chapter to have exactly one block')

        block, = blocks
        self.assertEquals(['The Remark'], block.css('h3').text,
                          'Expected block title to be visible.')

        self.assertEquals(
            '<p>Some <b>body</b> text</p>',
            block.css('.sl-text-wrapper').first.normalized_innerHTML)

    @browsing
    def test_latex_fields_available(self, browser):
        browser.login().open(self.chapter)
        factoriesmenu.add('Remark')

        form = browser.find('Title').parent('form')
        labels = form.field_labels

        self.assertIn('LaTeX code above content', labels)
        self.assertIn('LaTeX code beneath content', labels)

    @browsing
    def test_default_fields_not_visible(self, browser):
        browser.login().open(self.chapter)
        factoriesmenu.add('Remark')

        self.assertFalse(
            browser.find('Description'),
            '"Description" field should not be visible.')

        self.assertFalse(
            browser.find('Show Title'),
            '"Show Title" field should not be visible.')

        self.assertFalse(
            browser.find('Exclude from navigation'),
            '"Exclude from navigation" field should not be visible.')

        self.assertFalse(
            browser.find('Hide from table of contents'),
            '"Hide from table of contents" field should not be visible.')

    @browsing
    def test_remark_does_not_appear_in_navigation(self, browser):
        create(Builder('remark').titled('The Remark').within(self.chapter))

        browser.login().open(self.chapter)
        self.assertItemsEqual(
            ['The Book', 'The Chapter'],
            browser.css('.portletNavigationTree dd a').text)



from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.book.helpers import BookHelper
from ftw.book.interfaces import IBook
from ftw.testing import MockTestCase
from mocker import ANY
from zope.interface import directlyProvides


class TestUnitBookHelper(MockTestCase):
    """ Unittests for helper methods in BookHelper Class
    """

    def setUp(self):
        """ Setup test environment

        book
        |
        --- chapter1
        |
        --- chapter2
            |
            ---chapter3
            |  |
            |  ---chapter5
            |  |
            |  ---image
            |  |
            |  ---textblock1
            |  |
            |  ---textblock2
            |
            ---chapter4
        """
        super(TestUnitBookHelper, self).setUp()

        self._injection = {}

        # Helper object
        self.helper = BookHelper()

        #  Siteroot
        self.root = self.create_dummy()
        directlyProvides(self.root, IPloneSiteRoot)

        #  Book
        self.book = self.create_dummy()
        directlyProvides(self.book, [IBook, IFolderish])
        self.book = self.mocker.proxy(self.book, spec=False, count=False)
        self.expect(self.book.__parent__).result(self.root)
        self.expect(self.book.Title()).result('Book')
        self.mock_inject_settings(self.book)

        # Children of book
        self.chapter1 = self.create_dummy()
        directlyProvides(self.chapter1, IFolderish)
        self.chapter1 = self.mocker.proxy(
            self.chapter1, spec=False, count=False)
        self.expect(self.chapter1.__parent__).result(self.book)
        self.expect(self.chapter1.Title()).result('Chapter1')
        self.expect(self.chapter1.portal_type).result('Chapter')
        self.mock_inject_settings(self.chapter1)

        self.chapter2 = self.create_dummy()
        directlyProvides(self.chapter2, IFolderish)
        self.chapter2 = self.mocker.proxy(
            self.chapter2, spec=False, count=False)
        self.expect(self.chapter2.__parent__).result(self.book)
        self.expect(self.chapter2.Title()).result('Chapter2')
        self.expect(self.chapter2.portal_type).result('Chapter')
        self.mock_inject_settings(self.chapter2)

        #  Children of chapter 2
        self.chapter3 = self.create_dummy()
        directlyProvides(self.chapter3, IFolderish)
        self.chapter3 = self.mocker.proxy(
            self.chapter3, spec=False, count=False)
        self.expect(self.chapter3.__parent__).result(self.chapter2)
        self.expect(self.chapter3.Title()).result('Chapter3')
        self.expect(self.chapter3.portal_type).result('Chapter')
        self.mock_inject_settings(self.chapter3)

        self.chapter4 = self.create_dummy()
        directlyProvides(self.chapter4, IFolderish)
        self.chapter4 = self.mocker.proxy(
            self.chapter4, spec=False, count=False)
        self.expect(self.chapter4.__parent__).result(self.chapter2)
        self.expect(self.chapter4.Title()).result('Chapter4')
        self.expect(self.chapter4.portal_type).result('Chapter')
        self.mock_inject_settings(self.chapter4)

        #  Children of chapter 3
        self.chapter5 = self.create_dummy()
        directlyProvides(self.chapter5, IFolderish)
        self.chapter5 = self.mocker.proxy(
            self.chapter5, spec=False, count=False)
        self.expect(self.chapter5.__parent__).result(self.chapter3)
        self.expect(self.chapter5.Title()).result('Chapter5')
        self.expect(self.chapter5.portal_type).result('Chapter')
        self.mock_inject_settings(self.chapter5)

        self.image = self.mocker.mock(count=False)
        self.expect(self.image.__parent__).result(self.chapter3)
        self.expect(self.image.Title()).result('Image')
        self.expect(self.image.portal_type).result('Image')
        self.mock_inject_settings(self.image)

        self.textblock1 = self.mocker.mock(count=False)
        self.expect(self.textblock1.__parent__).result(self.chapter3)
        self.expect(self.textblock1.showTitle).result(True)
        self.expect(self.textblock1.Title()).result('Textblock1')
        self.expect(self.textblock1.title_or_id()).result('Textblock1')
        self.expect(self.textblock1.portal_type).result('Textblock')
        self.mock_inject_settings(self.textblock1, hideFromTOC=False)

        self.textblock2 = self.mocker.mock(count=False)
        self.expect(self.textblock2.__parent__).result(self.chapter3)
        self.expect(self.textblock2.__inner__).result(self.textblock2)
        self.expect(self.textblock2.showTitle).result(True)
        self.expect(self.textblock2.Title()).result('Textblock2')
        self.expect(self.textblock2.title_or_id()).result('Textblock2')
        self.expect(self.textblock2.portal_type).result('Textblock')
        self.mock_inject_settings(self.textblock2, hideFromTOC=False)

        # Folderish content
        self.expect(self.book.contentValues()).result(
            [self.chapter1, self.chapter2])
        self.expect(self.chapter1.contentValues()).result([])
        self.expect(self.chapter2.contentValues()).result(
            [self.chapter3, self.chapter4])
        self.expect(self.chapter3.contentValues()).result(
            [self.chapter5, self.image, self.textblock1, self.textblock2])
        self.expect(self.chapter4.contentValues()).result([])
        self.expect(self.chapter5.contentValues()).result([])

    def mock_inject_settings(self, mock, **settings):
        if mock not in self._injection:
            cache = self._injection[mock] = {'schema': self.stub()}
            schema = cache['schema']
            self.expect(mock.Schema()).result(schema).count(0, None)

            self.expect(schema.getField(ANY)).call(
                lambda name: cache.get(name, None))

        else:
            cache = self._injection[mock]

        for name, value in settings.items():
            field = self.stub()
            cache[name] = field
            self.expect(field.get(mock)).result(value)

    def test_generate_title(self):
        """ Test generate title
        """
        self.replay()

        self.assertEqual(self.helper(self.textblock1),
                         '<h4>2.1.2 Textblock1</h4>')
        self.assertEqual(self.helper(self.textblock2),
                         '<h4>2.1.3 Textblock2</h4>')

    def test_generate_title_respects_hideFromTOC(self):
        """ Test generate title
        """
        self.mock_inject_settings(self.textblock1, hideFromTOC=True)
        self.replay()

        # hideFromTOC is True
        self.assertEqual(self.helper(self.textblock1),
                         '<h4>Textblock1</h4>')
        self.assertEqual(self.helper(self.textblock2),
                         '<h4>2.1.2 Textblock2</h4>')

    def test_generated_title_result_is_consistent(self):
        self.mock_inject_settings(self.textblock1, hideFromTOC=True)
        self.replay()

        self.assertTrue(self.textblock1.Schema().getField(
                'hideFromTOC').get(self.textblock1))

        self.assertTrue(self.textblock1.Schema().getField(
                'hideFromTOC').get(self.textblock1))

        self.assertTrue(self.textblock1.Schema().getField(
                'hideFromTOC').get(self.textblock1))

        self.assertEqual(BookHelper()(self.textblock1),
                         '<h4>Textblock1</h4>')

        self.assertEqual(BookHelper()(self.textblock1),
                         '<h4>Textblock1</h4>')


    def test_generate_valid_hierarchy_h_tags(self):
        """ Test generate valid hierarchy h tags
        """
        self.replay()
        tags = self.helper.generate_valid_hierarchy_h_tags(self.chapter3)
        self.assertTrue(
            tags == '<h1>Book</h1>' +
                    '<h2>Chapter2</h2>' +
                    '<h3>Chapter3</h3>')

    def test_get_folder_position(self):
        """ Test get folder position
        """
        self.replay()
        position = self.helper.get_folder_position(self.textblock2)
        self.assertTrue(position == 3)

    def test_get_hierarchy_position(self):
        """ Test hierarchy position
        """
        self.replay()
        position = self.helper.get_hierarchy_position(self.textblock2)
        self.assertTrue(position == 4)



from ftw.book.interfaces import IHTMLBlock
from ftw.book.latex import utils
from ftw.pdfgenerator.view import MakoLaTeXView
from zope.component import adapts
from zope.interface import Interface


class HTMLBlockLaTeXView(MakoLaTeXView):
    adapts(IHTMLBlock, Interface, Interface)

    def render(self):
        latex = []

        if self.context.getShowTitle():
            latex.append(utils.get_latex_heading(self.context, self.layout))

        text = self.context.getText().strip()
        if len(text) > 0:
            latex.append(self.convert(text))

        latex.append('')
        return '\n'.join(latex)



from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book.browser.toc_tree import BookTocTree
from plone.app.layout.navigation.navtree import buildFolderTree
from zope.publisher.browser import BrowserView


class IndexView(BrowserView):

    template = ViewPageTemplateFile('templates/index_view.pt')

    def __init__(self, *args, **kwargs):
        super(IndexView, self).__init__(*args, **kwargs)
        self.tree = None

    def __call__(self):
        context = self.context

        query = self._build_query()

        raw_tree = buildFolderTree(context, obj=context, query=query)
        toc_tree = BookTocTree()
        tree = toc_tree(raw_tree)
        self.tree = tree
        return self.template()

    def _build_query(self):
        query = {
            'path': '/'.join(self.context.getPhysicalPath())}

        portal_properties = getToolByName(self.context, 'portal_properties')
        portal_catalog = getToolByName(self.context, 'portal_catalog')
        navtree_properties = getattr(portal_properties, 'navtree_properties')

        blacklist = navtree_properties.getProperty('metaTypesNotToList', ())
        all_types = portal_catalog.uniqueValuesFor('portal_type')
        query['portal_type'] = [t for t in all_types if t not in blacklist]

        return query



import os
from setuptools import setup, find_packages


version = '3.4.1'
maintainer = 'Jonas Baumann'


tests_require = [
    'Products.GenericSetup',
    'ftw.builder',
    'ftw.inflator',
    'ftw.publisher.core',
    'ftw.tabbedview',
    'ftw.testbrowser',
    'ftw.testing',
    'ftw.zipexport',
    'mocker',
    'plone.app.testing',
    'plone.browserlayer',
    'plone.mocktestcase',
    'plone.portlets',
    'plone.testing',
    'transaction',
    'unittest2',
    'zope.app.component',
    'zope.browser',
    'zope.configuration',
    'zope.i18n',
    'zope.traversing',
    'zope.viewlet',
    ]

extras_require = {
    'tests': tests_require,
    'tabbeview': [
        'ftw.tabbedview',
        'ftw.table'],
    'linguaplone': [
        'Products.LinguaPlone']}

setup(name='ftw.book',
      version=version,
      description='Produce books with Plone and export them in a high ' + \
          'quality PDF.',

      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw book pdf plone',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.book',

      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'AccessControl',
        'Acquisition',
        'BeautifulSoup!=4.0b',
        'Plone',
        'Products.ATContentTypes',
        'Products.Archetypes',
        'Products.CMFCore',
        'Products.CMFPlone',
        'Products.DataGridField>=1.9',
        'Products.GenericSetup',
        'Products.TinyMCE',
        'Products.statusmessages',
        'Zope2',
        'archetypes.schemaextender',
        'ftw.contentpage !=1.11.0, !=1.11.1',
        'ftw.pdfgenerator>=1.4',
        'ftw.profilehook',
        'ftw.upgrade',
        'lxml',
        'plone.app.contentmenu',
        'plone.app.layout',
        'plone.app.portlets',
        'plone.indexer',
        'plone.portlets',
        'setuptools',
        'simplelayout.base >= 4.0.0',
        'zope.annotation',
        'zope.component',
        'zope.dottedname',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
        ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book import _
from ftw.book.helpers import BookHelper
from ftw.pdfgenerator.html2latex.subconverters.table import TableConverter
from plone.app.layout.viewlets import ViewletBase
from simplelayout.base import viewlets
import re


class SimpleLayoutListingViewlet(viewlets.SimpleLayoutListingViewlet):

    render = ViewPageTemplateFile('listing.pt')
    helper = BookHelper()

    def get_valid_parent_h_tags(self):
        return self.helper.generate_valid_hierarchy_h_tags(self.context)


class TableValidationViewlet(ViewletBase):

    render = ViewPageTemplateFile('table_validation.pt')

    def __init__(self, *args, **kwargs):
        super(TableValidationViewlet, self).__init__(*args, **kwargs)
        self.errors = None

    def update(self):
        self.errors = self._validate()

    def _validate(self):
        errors = set()
        errors = self._validate_tables(errors)
        return errors

    def _validate_tables(self, errors):
        xpr = re.compile(TableConverter.pattern, re.DOTALL)
        start = 0

        html = self.context.getText()

        while True:
            match = xpr.search(html, start)
            if not match:
                return errors

            start = match.end()
            errors = self._validate_table_width(html, match, errors)

    def _validate_table_width(self, html, match, errors):
        table = TableConverter(None, match, html)
        table.parse()

        for column in table.columns:
            if not column.get_width():
                errors.add(
                    _(u'table_width_validation_warning',
                      u'Please specify the width of the table ' +
                      u'columns / cells.'))
                return errors

        return errors



from Acquisition import aq_inner, aq_parent
from ftw.book.interfaces import IBook
from ftw.book.interfaces import IChapter
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from mocker import ANY
from zope.component import getMultiAdapter
from zope.interface import alsoProvides


class TestChapterLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def test_converter(self):
        request = self.create_dummy()
        book = self.providing_stub([IBook])

        chapter = self.providing_mock([IChapter])
        self.expect(aq_parent(aq_inner(chapter))).result(book)
        self.expect(chapter.pretty_title_or_id()).result('chapter title')
        self.expect(chapter.listFolderContents()).result([])
        schema = self.stub()
        self.expect(chapter.Schema()).result(schema)
        self.expect(schema.getField(ANY)).result(None)

        layout = self.providing_mock([ILaTeXLayout])
        self.expect(layout.context).result(book).count(2)
        self.expect(layout.get_converter().convert('chapter title')).result(
            'converted chapter title')

        self.replay()

        view = getMultiAdapter((chapter, request, layout),
                               ILaTeXView)
        latex = view.render()

        self.assertEqual(latex, '\\chapter{converted chapter title}\n')

    def test_get_heading_counters_latex_with_book(self):
        request = self.create_dummy()
        book = self.providing_stub([IBook])
        chapter = self.providing_stub([IChapter])
        self.set_parent(chapter, book)

        layout = self.providing_stub([ILaTeXLayout])
        self.expect(layout.context).result(book)

        self.replay()

        view = getMultiAdapter((chapter, request, layout), ILaTeXView)

        # Since we are exporting the book and not the chapter directly
        # there is no need to reset the heading counters.
        self.assertEqual(view.get_heading_counters_latex(), '')

    def test_get_heading_counters_latex_with_chapter(self):
        request = self.create_dummy()

        book = self.providing_stub([IBook])
        chapter1 = self.providing_stub([IChapter])
        chapter2 = self.providing_stub([IChapter])
        chapter2a = self.providing_stub([IChapter])
        chapter2b = self.providing_stub([IChapter])

        self.expect(chapter1.portal_type).result('Chapter')
        self.expect(chapter2.portal_type).result('Chapter')
        self.expect(chapter2a.portal_type).result('Chapter')
        self.expect(chapter2b.portal_type).result('Chapter')

        self.expect(book.contentValues()).result([chapter1, chapter2])
        self.set_parent(chapter1, book)
        self.set_parent(chapter2, book)

        self.expect(chapter2.contentValues()).result(
            [chapter2a, chapter2b])
        self.set_parent(chapter2a, chapter2)
        self.set_parent(chapter2b, chapter2)

        self.expect(chapter1.contentValues()).result([])
        self.expect(chapter2a.contentValues()).result([])
        self.expect(chapter2b.contentValues()).result([])

        self.replay()

        # We are exporting the chapter directly, so there is no book / parent
        # chapter heading and we need to reset the heading counters.

        layout2a = self.create_dummy(context=chapter2a)
        alsoProvides(layout2a, ILaTeXLayout)
        view2a = getMultiAdapter((chapter2a, request, layout2a), ILaTeXView)
        self.assertEqual(view2a.get_heading_counters_latex(), '\n'.join((
                    r'\setcounter{chapter}{2}',
                    r''
                    )))

        layout2b = self.create_dummy(context=chapter2b)
        alsoProvides(layout2b, ILaTeXLayout)
        view2b = getMultiAdapter((chapter2b, request, layout2b), ILaTeXView)
        self.assertEqual(view2b.get_heading_counters_latex(), '\n'.join((
                    r'\setcounter{chapter}{2}',
                    r'\setcounter{section}{1}',
                    r''
                    )))



from Products.TinyMCE.browser.browser import TinyMCEBrowserView
import json


class BookTextBlockTinyMCEBrowserView(TinyMCEBrowserView):

    def jsonConfiguration(self, *args, **kwargs):
        """Return a customized configuration in JSON"""
        config = super(BookTextBlockTinyMCEBrowserView,
                       self).jsonConfiguration(*args, **kwargs)
        config = json.loads(config)
        if u'theme_advanced_buttons1' in config:
            # Plone >= 4.3
            config[u'theme_advanced_buttons1'] += (u',tinymce_keyword,'
                                                   u'tinymce_footnote,')
        else:
            # Plone <= 4.2
            config[u'buttons'].append(u'tinymce_keyword')
            config[u'buttons'].append(u'tinymce_footnote')
        return json.dumps(config)



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.pdfgenerator.browser.views import ExportPDFView
from ftw.pdfgenerator.utils import provide_request_layer
from ftw.book.interfaces import IAddRemarkLayer


class ExportBookPDFView(ExportPDFView):
    """Add the IAddRemark request layer if the user likes to show remarks in
    the exported pdf
    """

    index = ViewPageTemplateFile('templates/export_pdf.pt')

    def __call__(self):

        if self.request.get('embed_remarks', False):
            provide_request_layer(self.request, IAddRemarkLayer)

        if self.request.get('submitted', False):
            output = self.request.get('output', 'pdf')
            return self.export(output)

        return self.index()



from ftw.book.interfaces import IBookTextBlock
from ftw.book.latex import utils
from ftw.book.latex.utils import ImageLaTeXGenerator
from ftw.pdfgenerator.view import MakoLaTeXView
from simplelayout.base.interfaces import IBlockConfig
from zope.component import adapts
from zope.interface import Interface


class TextBlockLaTeXView(MakoLaTeXView):
    adapts(IBookTextBlock, Interface, Interface)

    def render(self):
        latex = []

        if self.context.getShowTitle():
            latex.append(utils.get_latex_heading(self.context, self.layout))

        text_latex = self.get_text_latex()
        floatable = text_latex and True or False
        image_latex = self.get_image_latex(floatable)

        latex.append(image_latex)
        latex.append(text_latex)

        return '\n'.join(latex)

    def get_text_latex(self):
        tex = ''
        text = self.context.getText().strip()
        if len(text) > 0:
            tex += self.convert(text)
            tex += '\n'
        return tex

    def get_image_latex(self, floatable):
        image = self.context.getImage()

        if not image or image.get_size() == 0:
            return ''

        image_layout = IBlockConfig(self.context).image_layout
        caption = self.context.getImageCaption()
        generator = ImageLaTeXGenerator(self.context, self.layout)

        return generator(image, image_layout, floatable=floatable,
                         caption=caption)



from Acquisition import aq_inner, aq_parent
from Products.Archetypes import atapi
from Products.Archetypes import public
from Products.CMFCore.utils import getToolByName
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import ISchemaExtender
from ftw.book import _
from ftw.book.interfaces import IBook
from ftw.book.latex.layouts import register_book_layout
from ftw.book.latex.utils import get_raw_image_data
from ftw.pdfgenerator.babel import get_preferred_babel_option_for_context
from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.layout.makolayout import MakoLayoutBase
from zope.component import adapts
from zope.dottedname.resolve import resolve
from zope.i18n import translate
from zope.interface import implements, Interface


class StringField(ExtensionField, public.StringField):
    pass


class TextField(ExtensionField, public.TextField):
    pass


class FileField(ExtensionField, public.FileField):
    pass


class IntegerField(ExtensionField, public.IntegerField):
    pass


class IDefaultBookLayoutSelectionLayer(Interface):
    """Request layer interface for selecting the default book layout.
    """

register_book_layout(IDefaultBookLayoutSelectionLayer,
                     _(u'Default layout'))


class DefaultBookLayoutExtender(object):
    """Schema extender, adding the layout-specific fields "release", "author"
    and "author_address" to the book when the default layout is selected.
    """

    adapts(IBook)
    implements(ISchemaExtender)

    fields = [
        StringField(
            name='release',
            default='',
            required=False,
            widget=atapi.StringWidget(
                label=_(u'book_label_release', default=u'Release'),
                )),

        StringField(
            name='author',
            default='',
            required=False,
            widget=atapi.StringWidget(
                label=_(u'book_label_author', default=u'Author'),
                )),

        TextField(
            name='author_address',
            default='',
            required=False,
            default_content_type='text/plain',
            allowable_content_types=('text/plain',),
            default_output_type='text/plain',

            widget=atapi.TextAreaWidget(
                label=_(u'book_label_author_address',
                        default=u'Author Address'),
                )),

        FileField(
            name='titlepage_logo',
            required=False,

            widget=atapi.FileWidget(
                label=_(u'book_label_titlepage_logo',
                        default=u'Titlepage logo'),
                description=_(u'book_help_titlepage_logo',
                              default=u'Upload an image or a PDF, which '
                              u'will be displayed on the titlepage'))),

        IntegerField(
            name='titlepage_logo_width',
            default=0,
            required=False,
            size=3,

            widget=atapi.IntegerWidget(
                label=_(u'book_label_titlepage_logo_width',
                        default=u'Titlepage logo width (%)'),
                description=_(u'book_help_titlepage_logo_width',
                              default=u'Width of the titlepage logo in '
                              u'percent of the content width.'),
                size=3,
                maxlength=3)),

        ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        # Checking whether the request provides the
        # IDefaultBookLayoutSelectionLayer interface does not work when
        # LinguaPlone is installed.
        # Looks like this method is called before
        # BookTraverse.publishTraverse() marks the request with the interface.

        if self.context.isTemporary():
            return []

        layout_layer_name = getattr(self.context, 'latex_layout', None)
        if layout_layer_name:
            layout_layer = resolve(layout_layer_name)
            if layout_layer == IDefaultBookLayoutSelectionLayer:
                return self.fields
        return []


class DefaultBookLayout(MakoLayoutBase):
    """A default book layout based on sphinx layout.
    """

    adapts(Interface, IDefaultBookLayoutSelectionLayer, IBuilder)

    template_directories = ['default_layout_templates']
    template_name = 'main.tex'

    def get_render_arguments(self):
        book = self.get_book()

        convert = self.get_converter().convert

        address = book.Schema().getField('author_address').get(book)
        address = convert(address.replace('\n', '<br />')).replace('\n', '')

        logo = book.Schema().getField('titlepage_logo').get(book)
        if logo and logo.data:
            logo_filename = 'titlepage_logo.jpg'
            self.get_builder().add_file(
                logo_filename,
                data=get_raw_image_data(logo.data))

            logo_width = book.Schema().getField(
                'titlepage_logo_width').get(book)
        else:
            logo_filename = False
            logo_width = 0

        args = {
            'context_is_book': self.context == book,
            'title': book.Title(),
            'use_titlepage': book.getUse_titlepage(),
            'logo': logo_filename,
            'logo_width': logo_width,
            'use_toc': book.getUse_toc(),
            'use_lot': book.getUse_lot(),
            'use_loi': book.getUse_loi(),
            'use_index': book.getUse_index(),
            'release': convert(book.Schema().getField('release').get(book)),
            'author': convert(book.Schema().getField('author').get(book)),
            'authoraddress': address,
            'babel': get_preferred_babel_option_for_context(self.context),
            'index_title': self.get_index_title(),
            }
        return args

    def get_book(self):
        obj = self.context
        while obj and not IBook.providedBy(obj):
            obj = aq_parent(aq_inner(obj))
        return obj

    def before_render_hook(self):
        self.use_package('inputenc', options='utf8', append_options=False)
        self.use_package('fontenc', options='T1', append_options=False)
        self.use_package('babel')
        self.use_package('times')
        self.use_package('fncychap', 'Sonny', append_options=False)
        self.use_package('longtable')
        self.use_package('sphinx')

        self.add_raw_template_file('sphinx.sty')
        self.add_raw_template_file('fncychap.sty')
        self.add_raw_template_file('sphinxftw.cls')
        self.add_raw_template_file('sphinxhowto.cls')
        self.add_raw_template_file('sphinxmanual.cls')

        # The sphinx document class requires graphicx and hyperref, so we
        # need to remove those packages from the document, otherwise it
        # would clash.
        self.remove_package('graphicx')
        self.remove_package('hyperref')

    def get_index_title(self):
        context_language_method = getattr(self.context, 'getLanguage', None)
        if context_language_method:
            language_code = context_language_method()

        else:
            ltool = getToolByName(self.context, 'portal_languages')
            language_code = ltool.getPreferredLanguage()

        return translate(_(u'title_index', default=u'Index'),
                         target_language=language_code)



from ftw.book.config import BOOK_LAYOUT_REGISTRY
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary


def register_book_layout(request_layer, title):
    """Registers a custom book layout, selectable in the book edit form.
    The `request_layer` interface of the select layout will be provided by
    the request when the PDF is exported. The layout adapter needs to adapt
    the `request_layer`. The `request_layer` should not be registered as
    normal browser layer.

    Arguments:
    request_layer -- a request layer interface, adapted by the layout
    title -- the title of the layout, displayed in the layout selection

    """
    dotted_name = '.'.join((request_layer.__module__, request_layer.__name__))
    BOOK_LAYOUT_REGISTRY[dotted_name] = title


class LayoutsVocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Returns a vocabulary with all known LaTeX layouts. A LaTeX layout
        is "registered" by subclassing IBookLaTeXLayoutSelectionLayer and
        providing an adapter, which uses the new interface as request
        discriminator.
        """
        terms = []

        for iface_dotted_name, title in BOOK_LAYOUT_REGISTRY.items():
            value = iface_dotted_name
            terms.append(SimpleVocabulary.createTerm(value, value, title))

        return SimpleVocabulary(terms)
